@@ -49,27 +49,7 @@ aim_tlv_read(GSList *list, ByteStream *bs)
 	type = byte_stream_get16(bs);
 	length = byte_stream_get16(bs);
 
-#if 0
-	/*
-	 * This code hasn't been needed in years.  It's been commented
-	 * out since 2003, at the latest.  It seems likely that it was
-	 * just a bug in their server code that has since been fixed.
-	 * In any case, here's the orignal comment, kept for historical
-	 * purposes:
-	 *
-	 * Okay, so now AOL has decided that any TLV of
-	 * type 0x0013 can only be two bytes, despite
-	 * what the actual given length is.  So here
-	 * we dump any invalid TLVs of that sort.  Hopefully
-	 * there's no special cases to this special case.
-	 *   - mid (30jun2000)
-	 */
-	if ((type == 0x0013) && (length != 0x0002)) {
-		length = 0x0002;
-		return list;
-	}
-#endif
-	if (length > byte_stream_empty(bs)) {
+	if (length > byte_stream_bytes_left(bs)) {
 		aim_tlvlist_free(list);
 		return NULL;
 	}
@@ -108,7 +88,7 @@ GSList *aim_tlvlist_read(ByteStream *bs)
 {
 	GSList *list = NULL;
 
-	while (byte_stream_empty(bs) > 0) {
+	while (byte_stream_bytes_left(bs) > 0) {
 		list = aim_tlv_read(list, bs);
 		if (list == NULL)
 			return NULL;
@@ -142,7 +122,7 @@ GSList *aim_tlvlist_readnum(ByteStream *bs, guint16 num)
 {
 	GSList *list = NULL;
 
-	while ((byte_stream_empty(bs) > 0) && (num != 0)) {
+	while ((byte_stream_bytes_left(bs) > 0) && (num != 0)) {
 		list = aim_tlv_read(list, bs);
 		if (list == NULL)
 			return NULL;
@@ -177,7 +157,7 @@ GSList *aim_tlvlist_readlen(ByteStream *bs, guint16 len)
 {
 	GSList *list = NULL;
 
-	while ((byte_stream_empty(bs) > 0) && (len > 0)) {
+	while ((byte_stream_bytes_left(bs) > 0) && (len > 0)) {
 		list = aim_tlv_read(list, bs);
 		if (list == NULL)
 			return NULL;
@@ -391,6 +371,17 @@ int aim_tlvlist_add_str(GSList **list, const guint16 type, const char *value)
 	return aim_tlvlist_add_raw(list, type, strlen(value), (guint8 *)value);
 }
 
+static int
+count_caps(guint64 caps)
+{
+	int set_bits = 0;
+	while (caps) {
+		set_bits += caps & 1;
+		caps >>= 1;
+	}
+	return set_bits;
+}
+
 /**
  * Adds a block of capability blocks to a TLV chain. The bitfield
  * passed in should be a bitwise %OR of any of the %AIM_CAPS constants:
@@ -409,23 +400,24 @@ int aim_tlvlist_add_str(GSList **list, const guint16 type, const char *value)
  */
 int aim_tlvlist_add_caps(GSList **list, const guint16 type, const guint64 caps, const char *mood)
 {
-	guint8 buf[256]; /* TODO: Don't use a fixed length buffer */
 	ByteStream bs;
+	guint32 bs_size;
 	guint8 *data;
 
 	if (caps == 0)
 		return 0; /* nothing there anyway */
 
-	byte_stream_init(&bs, buf, sizeof(buf));
-
-	byte_stream_putcaps(&bs, caps);
-	
-	/* adding of custom icon GUID */
 	data = icq_get_custom_icon_data(mood);
+	bs_size = 16*(count_caps(caps) + (data != NULL ? 1 : 0));
+
+	byte_stream_new(&bs, bs_size);
+	byte_stream_putcaps(&bs, caps);
+
+	/* adding of custom icon GUID */
 	if (data != NULL)
 		byte_stream_putraw(&bs, data, 16);
 
-	return aim_tlvlist_add_raw(list, type, byte_stream_curpos(&bs), buf);
+	return aim_tlvlist_add_raw(list, type, byte_stream_curpos(&bs), bs.data);
 }
 
 /**
@@ -668,7 +660,7 @@ int aim_tlvlist_write(ByteStream *bs, GSList **list)
 	/* do an initial run to test total length */
 	goodbuflen = aim_tlvlist_size(*list);
 
-	if (goodbuflen > byte_stream_empty(bs))
+	if (goodbuflen > byte_stream_bytes_left(bs))
 		return 0; /* not enough buffer */
 
 	/* do the real write-out */
